﻿
using System.ComponentModel;

namespace QuickType {
    using System;
    using System.Collections.Generic;

    using System.Globalization;
    using Newtonsoft.Json;
    using Newtonsoft.Json.Converters;

    public partial class Media {
        [JsonProperty("track-count")]
        public long TrackCount { get; set; }

        [JsonProperty("title")]
        public string Title { get; set; }

        [JsonProperty("format")]
        public string Format { get; set; }

        [JsonProperty("position")]
        public long Position { get; set; }

        [JsonProperty("discs")]
        public Welcome[] Discs { get; set; }

        [JsonProperty("track-offset")]
        public long TrackOffset { get; set; }

        [JsonProperty("format-id")]
        public Guid FormatId { get; set; }

        [JsonProperty("tracks")]
        public Track[] Tracks { get; set; }

        [JsonProperty("data-tracks", NullValueHandling = NullValueHandling.Ignore)]
        public Track[] DataTracks { get; set; }
    }

    public partial class Release {
        [JsonProperty("media")]
        public Media[] Media { get; set; }

        [JsonProperty("id")]
        public Guid Id { get; set; }

        [JsonProperty("barcode")]
        public string Barcode { get; set; }

        [JsonProperty("asin")]
        public string Asin { get; set; }

        [JsonProperty("packaging")]
        public string Packaging { get; set; }

        [JsonProperty("title")]
        public string Title { get; set; }

        [JsonProperty("text-representation")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public TextRepresentation TextRepresentation { get; set; }

        [JsonProperty("status")]
        public string Status { get; set; }

        [JsonProperty("country")]
        public string Country { get; set; }

        [JsonProperty("release-events")]
        public ReleaseEvent[] ReleaseEvents { get; set; }

        [JsonProperty("disambiguation")]
        public string Disambiguation { get; set; }

        [JsonProperty("cover-art-archive")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public CoverArtArchive CoverArtArchive { get; set; }

        [JsonProperty("status-id")]
        public Guid StatusId { get; set; }

        [JsonProperty("quality")]
        public string Quality { get; set; }

        [JsonProperty("packaging-id")]
        public Guid? PackagingId { get; set; }

        [JsonProperty("date")]
        [JsonConverter(typeof(DateConverter))]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public Date Date { get; set; }
    }

    public partial class Welcome {
        [JsonProperty("offset-count")]
        public long OffsetCount { get; set; }

        [JsonProperty("sectors")]
        public long Sectors { get; set; }

        [JsonProperty("releases", NullValueHandling = NullValueHandling.Ignore)]
        public Release[] Releases { get; set; }

        [JsonProperty("offsets")]
        public long[] Offsets { get; set; }

        [JsonProperty("id")]
        public string Id { get; set; }
    }

    public partial class Track {
        [JsonProperty("recording")]
        public Recording Recording { get; set; }

        [JsonProperty("position")]
        public long Position { get; set; }

        [JsonProperty("title")]
        public string Title { get; set; }

        [JsonProperty("id")]
        public Guid Id { get; set; }

        [JsonProperty("number")]
        [JsonConverter(typeof(ParseStringConverter))]
        public long Number { get; set; }

        [JsonProperty("length")]
        public long Length { get; set; }
    }

    public partial class Recording {
        [JsonProperty("disambiguation")]
        [JsonConverter(typeof(DisambiguationConverter))]
        public Disambiguation Disambiguation { get; set; }

        [JsonProperty("id")]
        public Guid Id { get; set; }

        [JsonProperty("title")]
        public string Title { get; set; }

        [JsonProperty("video")]
        public bool Video { get; set; }

        [JsonProperty("length")]
        public long Length { get; set; }

        [JsonProperty("first-release-date")]
        [JsonConverter(typeof(DateConverter))]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public Date FirstReleaseDate { get; set; }
    }

    public partial class CoverArtArchive {
        [JsonProperty("darkened")]
        public bool Darkened { get; set; }

        [JsonProperty("back")]
        public bool Back { get; set; }

        [JsonProperty("artwork")]
        public bool Artwork { get; set; }

        [JsonProperty("front")]
        public bool Front { get; set; }

        [JsonProperty("count")]
        public long Count { get; set; }
    }

    public partial class ReleaseEvent {
        [JsonProperty("date")]
        [JsonConverter(typeof(DateConverter))]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public Date Date { get; set; }

        [JsonProperty("area")]
        public Area Area { get; set; }
    }

    public partial class Area {
        [JsonProperty("iso-3166-1-codes")]
        public string[] Iso31661Codes { get; set; }

        [JsonProperty("type")]
        public object Type { get; set; }

        [JsonProperty("id")]
        public Guid Id { get; set; }

        [JsonProperty("disambiguation")]
        public string Disambiguation { get; set; }

        [JsonProperty("name")]
        public string Name { get; set; }

        [JsonProperty("type-id")]
        public object TypeId { get; set; }

        [JsonProperty("sort-name")]
        public string SortName { get; set; }
    }

    public partial class TextRepresentation {
        [JsonProperty("script")]
        public string Script { get; set; }

        [JsonProperty("language")]
        public string Language { get; set; }
    }

    public enum Disambiguation { Empty, Live };

    public partial struct Date {
        public DateTimeOffset? DateTime;
        public long? Integer;

        public static implicit operator Date(DateTimeOffset DateTime) => new Date { DateTime = DateTime };
        public static implicit operator Date(long Integer) => new Date { Integer = Integer };
    }

    public partial class Welcome {
        public static Welcome FromJson(string json) => JsonConvert.DeserializeObject<Welcome>(json, QuickType.Converter.Settings);
    }

    public static class Serialize {
        public static string ToJson(this Welcome self) => JsonConvert.SerializeObject(self, QuickType.Converter.Settings);
    }

    internal static class Converter {
        public static readonly JsonSerializerSettings Settings = new JsonSerializerSettings {
            MetadataPropertyHandling = MetadataPropertyHandling.Ignore,
            DateParseHandling = DateParseHandling.None,
            Converters =
            {
                DateConverter.Singleton,
                DisambiguationConverter.Singleton,
                new IsoDateTimeConverter { DateTimeStyles = DateTimeStyles.AssumeUniversal }
            },
        };
    }

    internal class DateConverter : JsonConverter {
        public override bool CanConvert(Type t) => t == typeof(Date) || t == typeof(Date?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer) {
            switch (reader.TokenType) {
                case JsonToken.String:
                case JsonToken.Date:
                    var stringValue = serializer.Deserialize<string>(reader);
                    DateTimeOffset dt;
                    if (DateTimeOffset.TryParse(stringValue, out dt)) {
                        return new Date { DateTime = dt };
                    }
                    long l;
                    if (Int64.TryParse(stringValue, out l)) {
                        return new Date { Integer = l };
                    }
                    break;
            }
            throw new Exception("Cannot unmarshal type Date");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer) {
            var value = (Date)untypedValue;
            if (value.DateTime != null) {
                serializer.Serialize(writer, value.DateTime.Value.ToString("o", System.Globalization.CultureInfo.InvariantCulture));
                return;
            }
            if (value.Integer != null) {
                serializer.Serialize(writer, value.Integer.Value.ToString());
                return;
            }
            throw new Exception("Cannot marshal type Date");
        }

        public static readonly DateConverter Singleton = new DateConverter();
    }

    internal class ParseStringConverter : JsonConverter {
        public override bool CanConvert(Type t) => t == typeof(long) || t == typeof(long?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer) {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<string>(reader);
            long l;
            if (Int64.TryParse(value, out l)) {
                return l;
            }
            throw new Exception("Cannot unmarshal type long");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer) {
            if (untypedValue == null) {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (long)untypedValue;
            serializer.Serialize(writer, value.ToString());
            return;
        }

        public static readonly ParseStringConverter Singleton = new ParseStringConverter();
    }

    internal class DisambiguationConverter : JsonConverter {
        public override bool CanConvert(Type t) => t == typeof(Disambiguation) || t == typeof(Disambiguation?);

        public override object ReadJson(JsonReader reader, Type t, object existingValue, JsonSerializer serializer) {
            if (reader.TokenType == JsonToken.Null) return null;
            var value = serializer.Deserialize<string>(reader);
            switch (value) {
                case "":
                    return Disambiguation.Empty;
                case "live":
                    return Disambiguation.Live;
            }
            throw new Exception("Cannot unmarshal type Disambiguation");
        }

        public override void WriteJson(JsonWriter writer, object untypedValue, JsonSerializer serializer) {
            if (untypedValue == null) {
                serializer.Serialize(writer, null);
                return;
            }
            var value = (Disambiguation)untypedValue;
            switch (value) {
                case Disambiguation.Empty:
                    serializer.Serialize(writer, "");
                    return;
                case Disambiguation.Live:
                    serializer.Serialize(writer, "live");
                    return;
            }
            throw new Exception("Cannot marshal type Disambiguation");
        }

        public static readonly DisambiguationConverter Singleton = new DisambiguationConverter();
    }
}
